/**********************************************************************
 * File vbsu.c
 *
 * call the given script with root rights
 *
 * In order to make this secure, the script files must be
 * owned by root:vboxusers and be have the right r-x------ and the
 * user  must be a member of the vboxuser group
 *
 * this file must be owned by root and the right must be
 * set uid bit must be set. 
 *
 **********************************************************************/
 
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <pwd.h>
#include <grp.h>

#define ALLOWED_GROUP "vboxusers"

/**********************************************************************
 *
 * Function isMember()
 *
 *   Check if the user is a member of the group vboxusers and put
 *   this group id into *gid
 *
 * Out: gid_t *gid
 *
 * Return: 1 if user is a member else 0
 *
 *********************************************************************/

static int isMember(gid_t *gid)
{
   uid_t uid = getuid(); /* where we are */
   struct passwd  *pwd;
   struct group   *grp;
   char          **member;
   
   /* look for our passwd entry (and user name) */
   while ((pwd = getpwent()))
   {
      if ( pwd->pw_uid == uid )
      {
         /* check if we are member of vboxusers group */
         while ((grp=getgrent()))
         {
            if ( strcmp(grp->gr_name, ALLOWED_GROUP) == 0 )
            {
               /* check member list */
               member = grp->gr_mem;
               while(*member)
               {
                  if ( strcmp(*member, pwd->pw_name) == 0 )
                  {
                     *gid = grp->gr_gid;
                     return 1;
                  }
                  member++;
               }
            }
         }
         break;
      }
   }
   return 0;
}

int main(int argc, char **argv)
{
   struct stat status;
   int    error = 0;
   char   *nm;
   gid_t   gid = 0;

   if ( !isMember(&gid) )
   {
      fprintf(stderr,"%s: Sorry you have no rights\n",argv[0]);
      exit(1);
   }
   
   /* check for arguments */
   if ( argc > 1 )
   {
      /* the first one must be the script name and
       * the rights of the scripz must ne OK:
       * owner root read and execute right only for
       * the owner.
       */

      if ( stat(argv[1], &status) == 0 )
      {
         if ( S_ISREG(status.st_mode )
              &&
              (status.st_mode & (S_IRWXO|S_IRWXG|S_IRWXU)) == (S_IRUSR|S_IXUSR)
              &&
              status.st_uid  == 0
              &&
              status.st_gid == gid
            )
         {
            /* this shall be OK, become root */
            setuid(0);
            seteuid(0);
            /* and modify the first argument */
            nm = argv[0];
            argv[0] = "/bin/sh";
            /* execute the script */
            error = execv(argv[0], argv);
            if ( error )
               perror(nm);
         }
         else
         {
            fprintf(stderr, "%s: No right to do this\n", argv[0]);
            error = 1;
         }
      }
      else
      {
          /* file not found ? */
         perror(argv[0]);
         error = 1;
      }
   }
   else
   {
      printf("Syntax: %s script [args...] \n",argv[0]);
      error = 1;
   }

   return error;
}
